@@ -9,8 +9,6 @@ from qmpy.utils import *
 
 logger = logging.getLogger(__name__)
 
-#__all__ = ['get_pdf']
-
 class PDF(object):
     """
     Container class for a Pair-distribution function.
@@ -23,20 +21,22 @@ class PDF(object):
 
     def __init__(self, structure, limit=6):
         elements = structure.comp.keys()
-        pairs = itertools.product(elements, elements)
+        pairs = itertools.combinations_with_replacement(elements, r=2)
         self.pairs = [ self.get_pair(pair) for pair in pairs ]
         self.distances = dict((p, []) for p in self.pairs)
         self.weights = dict((p, []) for p in self.pairs)
-        self.sites = list(structure.uniq_sites)
+
+        structure = structure.copy()
+        structure.reduce()
+        structure.symmetrize()
+        self.structure = structure
+        self.cell = self.structure.cell
+        self.uniq = self.structure.uniq_sites
+        self.sites = self.structure.sites
         self.limit = limit
-        if limit:
-            # make a structure large enough to contain all atoms `limit` away
-            # structure.reduce()
-            lp = structure.lat_params
-            limits = [ int(np.ceil(limit/lp[i])) for i in range(3) ]
-            self.structure = structure.t([ l+1 for l in limits], in_place=False)
-        else:
-            self.structure = structure.copy()
+        self.limit2 = limit**2
+        lp = structure.find_lattice_points_within_distance(limit)
+        self.lattice_points = np.array([ np.dot(p, self.cell) for p in lp ])
 
     def get_pair(self, pair):
         return tuple(sorted(pair))
@@ -47,24 +47,25 @@ class PDF(object):
         Returns a dict of (atom1, atom2):[list of distances].
 
         """
-        for s1 in self.sites:
-            c1 = self.structure.inv.T.dot(s1.cart_coord)
-            for a2 in self.structure.atoms:
-                dist = self.structure.get_distance(c1, a2, wrap_self=False)
-                # returns None if distance is greater than limit
-                if dist is None:
-                    continue
-                if self.limit:
-                    if dist > self.limit:
+        for s1 in self.uniq:
+            for s2 in self.sites:
+                _dist = s1.cart_coord - s2.cart_coord
+                for lp in self.lattice_points:
+                    dist = _dist + lp
+                    if any([ abs(p) > self.limit for p in dist ]):
                         continue
-                if abs(dist) < 0.001:
-                    dist = min(self.structure.lat_params[:3])
+                    dist2 = np.dot(dist, dist)
+                    if dist2 < 1e-6:
+                        continue
+                    if dist2 > self.limit2:
+                        continue
+                    distance = dist2**0.5
 
-                for a1 in s1.atoms:
-                    pair = self.get_pair((a1.element_id, a2.element_id))
-                    self.distances[pair] += [dist]
-                    w = s1.multiplicity*a1.occupancy*a2.occupancy
-                    self.weights[pair] += [float(w)]
+                    for a1, a2 in itertools.product(s1, s2):
+                        pair = self.get_pair([a1.element_id, a2.element_id])
+                        w = s1.multiplicity*a1.occupancy*a2.occupancy
+                        self.distances[pair].append(float(distance))
+                        self.weights[pair].append(float(w))
 
     def plot(self, smearing=0.1):
         renderer = Renderer()
