@@ -32,6 +32,9 @@ class PhaseData(object):
     def __init__(self):
         self.clear()
 
+    def __str__(self):
+        return '%d Phases' % len(self.phases)
+
     @property
     def phases(self):
         """
@@ -140,7 +143,7 @@ class PhaseData(object):
         from qmpy.materials.formation_energy import FormationEnergy
         from qmpy.materials.element import Element
         logger.debug('Loading Phases from the OQMD')
-        data = FormationEnergy.objects.exclude(delta_e=None, composition=None)
+        data = FormationEnergy.objects.all()
 
         if fit:
             data = data.filter(fit=fit)
@@ -274,17 +277,19 @@ class Phase(object):
     """
     A Phase object is a point in composition-energy space. 
 
-    Examples:
-    >>> p = Phase('Fe2O3', -1.64, per_atom=True)
-    >>> p2 = Phase('Fe2O3', -8.2, per_atom=False)
-    >>> p3 = Phase({'Fe':0.4, 'O':0.6}, -1.64)
-    >>> p4 = Phase({'Fe':6, 'O':9}, -24.6, per_atom=False)
-    >>> p == p2
-    True
-    >>> p2 == p3
-    True
-    >>> p3 == p4
-    True
+    Examples::
+
+        >>> p = Phase('Fe2O3', -1.64, per_atom=True)
+        >>> p2 = Phase('Fe2O3', -8.2, per_atom=False)
+        >>> p3 = Phase({'Fe':0.4, 'O':0.6}, -1.64)
+        >>> p4 = Phase({'Fe':6, 'O':9}, -24.6, per_atom=False)
+        >>> p == p2
+        True
+        >>> p2 == p3
+        True
+        >>> p3 == p4
+        True
+
     """
 
     id = None
@@ -318,11 +323,6 @@ class Phase(object):
         else:
             self.energy = energy
 
-        if len(self.comp) == 1 and not total:
-            self._energy = 0.0
-            self._total_energy = 0.0
-            self._energy_pfu = 0.0
-
     @staticmethod
     def from_phases(phase_dict):
         """
@@ -351,14 +351,17 @@ class Phase(object):
     def natoms(self):
         return sum(self.nom_comp.values())
 
-    def __repr__(self):
+    def __str__(self):
         if self.description:
-            return '<Phase {name} ({description}): {energy:0.3g}>'.format(
+            return '{name} ({description}): {energy:0.3g}'.format(
                     name=self.name, energy=self.energy, description=self.description)
         else:
-            return '<Phase {name} : {energy:0.3g}>'.format(
+            return '{name} : {energy:0.3g}'.format(
                     name=self.name, energy=self.energy)
 
+    def __repr__(self):
+        return '<Phase %s>' % self
+
     def __eq__(self,other):
         """
         Phases are defined to be equal if they have the same composition and an
@@ -543,17 +546,19 @@ class Phase(object):
         default it just returns the phase's ground state energy.
         """
         #global environment
-        return self._energy
+        return self.energy
 
     def fraction(self, comp):
         """
         Returns a composition dictionary with the specified composition pulled
         out as 'var'. 
 
-        Example:
-        >>> phase = Phase(composition={'Fe':1, 'Li':5, 'O'8'}, energy=-1)
-        >>> phase.fraction('Li2O')
-        defaultdict(<type 'float'>, {'var': 2.5, 'Fe': 1, 'O': 5.5, 'Li': 0.0})
+        Examples::
+
+            >>> phase = Phase(composition={'Fe':1, 'Li':5, 'O':8}, energy=-1)
+            >>> phase.fraction('Li2O')
+            defaultdict(<type 'float'>, {'var': 2.5, 'Fe': 1, 'O': 5.5, 'Li': 0.0})
+
         """
         if isinstance(comp, Phase):
             comp = comp.unit_comp
