@@ -76,7 +76,7 @@ class PhaseSpace(object):
             if bounds:
                 self.load(**kwargs)
         else:
-            self.data = data
+            self.data = data.get_phase_data(self.space)
 
     def __repr__(self):
         if self.bounds is None:
@@ -101,7 +101,7 @@ class PhaseSpace(object):
         if isinstance(bounds, basestring):
             bounds = re.sub('[-,_]', ' ', bounds)
             bounds = [ unit_comp(parse_comp(b)) for b in bounds.split()]
-        elif isinstance(bounds, list):
+        elif isinstance(bounds, (list,set)):
             bounds = [ {elt:1} for elt in bounds ]
         elif isinstance(bounds, dict):
             bounds = [ {elt:1} for elt in bounds ]
@@ -438,9 +438,9 @@ class PhaseSpace(object):
         """
         if self._spaces:
             return self._spaces
-        spaces = [ p.space for p in self.phase_dict.values() ]
-        spaces = set([ frozenset(space) for space in spaces if not 
-                any( space < space2 for space2 in spaces ) ])
+        spaces = set([ frozenset(p.space) for p in self.phase_dict.values() ])
+        spaces = [ space for space in spaces if not 
+                any([ space < space2 for space2 in spaces ])]
         self._spaces = list(map(list, spaces))
         return self._spaces
 
@@ -828,7 +828,10 @@ class PhaseSpace(object):
         while facets:
             facet = facets.pop(0)
             done_list.append(sorted(facet))
-            phases, E = self.get_minima(self.phase_dict.values(), facet)
+            try:
+                phases, E = self.get_minima(self.phase_dict.values(), facet)
+            except:
+                continue
             p = phase.Phase.from_phases(phases)
             if p in self.phases:
                 p = self.phase_dict[p.name]
@@ -899,8 +902,8 @@ class PhaseSpace(object):
 
         if pulp.GUROBI().available():
             prob.solve(pulp.GUROBI(msg=False))
-        if pulp.COINMP_DLL().available():
-            prob.solve(pulp.COINMP_DLL())
+        if pulp.COIN_CMD().available():
+            prob.solve(pulp.COIN_CMD())
         else:
             prob.solve()
 
@@ -932,8 +935,10 @@ class PhaseSpace(object):
         prob += sum([ bvars[b] for b in bounds ]) == 1, \
                 'sum of bounds must be 1'
 
-        if pulp.GUROBI().available():
-            prob.solve(pulp.GUROBI(msg=False))
+        #if pulp.GUROBI().available():
+        #    prob.solve(pulp.GUROBI(msg=False))
+        if pulp.COIN_CMD().available():
+            prob.solve(pulp.COIN_CMD())
         if pulp.COINMP_DLL().available():
             prob.solve(pulp.COINMP_DLL())
         else:
@@ -956,19 +961,19 @@ class PhaseSpace(object):
         Compute the energy difference between the formation energy of a Phase,
         and the energy of the convex hull in the absence of that phase. 
         """
-        if self.phase_dict[p.name] != p:
-            stable = self.phase_dict[p.name]
-            p.stability = p.energy - stable.energy
-        elif len(p.comp) == 1:
+        #if self.phase_dict[p.name] != p:
+        #    stable = self.phase_dict[p.name]
+        #    p.stability = p.energy - stable.energy
+        if len(p.comp) == 1:
             stable = self.phase_dict[p.name]
             p.stability = p.energy - stable.energy
         else:
-            phases = self.phase_dict.values()
-            phases.remove(p)
+            phases = [ pp for pp in self.phase_dict.values() if 
+                    p.name != pp.name ]
             energy, gclp_phases = self.gclp(p.unit_comp, phases=phases)
             p.stability = p.energy - energy
 
-    def compute_stabilities(self, phases=None, save=False):
+    def compute_stabilities(self, phases=None, save=False, new_only=False):
         """
         Calculate the stability for every Phase.
 
@@ -978,16 +983,30 @@ class PhaseSpace(object):
 
             save:
                 If True, save the value for stability to the database. 
+
+            new_only:
+                If True, only compute the stability for Phases which did not
+                import a stability from the OQMD. False by default.
         """
         from qmpy.analysis.vasp.calculation import Calculation
         if phases is None:
             phases = self.phases
         for p in phases:
-            if not p.stability:
-                self.compute_stability(p)
-                qs = qmpy.FormationEnergy.objects.filter(id=p.id)
-                if save:
-                    qs.update(stability=phase.stability)
+            if new_only:
+                if not p.stability is None:
+                    continue
+            self.compute_stability(p)
+            qs = qmpy.FormationEnergy.objects.filter(id=p.id)
+            if save:
+                qs.update(stability=p.stability)
+
+    def save_tie_lines(self):
+        """
+        Save all tie lines in this PhaseSpace to the OQMD. Stored in
+        Formation.equilibrium
+        """
+        for p1, p2 in self.tie_lines:
+            p1.formation.equilibrium.add(p2.formation)
 
     def compute_formation_energies(self):
         """
@@ -1008,6 +1027,7 @@ class PhaseSpace(object):
     renderer = None
     @property
     def phase_diagram(self, **kwargs):
+        """Renderer of a phase diagram of the PhaseSpace"""
         if self.renderer is None:
             self.get_phase_diagram(**kwargs)
         return self.renderer
@@ -1460,8 +1480,10 @@ class PhaseSpace(object):
                     'identical %s composition on both sides' % celt
         prob += sum([ rvars[p] for p in phases ]) == 1
         
-        if pulp.GUROBI().available():
-            prob.solve(pulp.GUROBI(msg=False))
+        #if pulp.GUROBI().available():
+        #    prob.solve(pulp.GUROBI(msg=False))
+        if pulp.COIN_CMD().available():
+            prob.solve(pulp.COIN_CMD())
         if pulp.COINMP_DLL().available():
             prob.solve(pulp.COINMP_DLL())
         else:
