@@ -1,11 +1,5 @@
 # qmpy/analysis/vasp/calculation.py
 
-"""
-The Calculation class is the container for all information about a calculation,
-and is used to read/write/create calculation directories.
-
-"""
-
 import os
 import copy
 import json
@@ -200,8 +194,8 @@ class Calculation(models.Model):
         self.potential_set = self.potentials
         self.element_set = self.elements
         self.meta_data = self.error_objects
-        if self.formation:
-            self.formationenergy_set.add(self.formation)
+        if not self.formation is None:
+            self.formation.save()
 
     # django caching
     _potentials = None
@@ -432,7 +426,7 @@ class Calculation(models.Model):
                 for atom, mom in zip(self.input.atoms, real_moms):
                     atom.magmom = float(mom)
                     if atom.id is not None:
-                        atom.save()
+                        Atom.objects.filter(id=atom.id).update(magmom=mom)
         self.settings = settings
 
     def get_kpoints(self):
@@ -794,16 +788,19 @@ class Calculation(models.Model):
         self.bands = np.array(bands)
 
     def read_outcar_results(self):
-        self.read_natoms()
-        self.read_convergence()
-        elts = self.read_elements()
-        energies = self.read_energies()
-        lattice_vectors = self.read_lattice_vectors()
-        stresses = self.read_stresses()
-        positions = self.read_positions()
-        forces = self.read_forces()
-        magmoms = self.read_magmoms()
-        charges = self.read_charges()
+        try:
+            self.read_natoms()
+            self.read_convergence()
+            elts = self.read_elements()
+            energies = self.read_energies()
+            lattice_vectors = self.read_lattice_vectors()
+            stresses = self.read_stresses()
+            positions = self.read_positions()
+            forces = self.read_forces()
+            magmoms = self.read_magmoms()
+            charges = self.read_charges()
+        except:
+            raise VaspError("Failed to read results")
 
         if len(self.energies) > 0:
             self.energy = self.energies[-1]
@@ -812,24 +809,27 @@ class Calculation(models.Model):
             self.magmom_pa = self.magmom/self.natoms
 
         if self.nsteps > 0:
-            output = strx.Structure()
-            output.total_energy = energies[-1]
-            output.cell = lattice_vectors[-1]
-            output.stresses = stresses[-1]
-            inv = numpy.linalg.inv(output.cell).T
-            atoms = []
-            for coord, forces, charge, magmom, elt in zip(positions[-1], 
-                                                          forces[-1],
-                                                          charges[-1],
-                                                          magmoms[-1],
-                                                          elts):
-                a = Atom(element_id=elt, charge=charge, magmom=magmom)
-                a.coord = np.dot(inv, coord)
-                a.forces = forces
-                atoms.append(a)
-            output.atoms = atoms
-            self.output = output
-            self.output.set_label(self.label)
+            try:
+                output = strx.Structure()
+                output.total_energy = energies[-1]
+                output.cell = lattice_vectors[-1]
+                output.stresses = stresses[-1]
+                inv = numpy.linalg.inv(output.cell).T
+                atoms = []
+                for coord, forces, charge, magmom, elt in zip(positions[-1], 
+                                                              forces[-1],
+                                                              charges[-1],
+                                                              magmoms[-1],
+                                                              elts):
+                    a = Atom(element_id=elt, charge=charge, magmom=magmom)
+                    a.coord = np.dot(inv, coord)
+                    a.forces = forces
+                    atoms.append(a)
+                output.atoms = atoms
+                self.output = output
+                self.output.set_label(self.label)
+            except:
+                raise VaspError("Improperly read results")
 
     def read_convergence(self):
         self.get_outcar()
@@ -1046,6 +1046,8 @@ class Calculation(models.Model):
 
     def read_outcar_started(self):
         self.get_outcar()
+        if not self.outcar:
+            return False
         if len(self.outcar) < 5:
             return False
         found_inputs = [False, False, False, False]
@@ -1066,8 +1068,6 @@ class Calculation(models.Model):
 
     def read_outcar(self):
         self.get_outcar()
-        if not self.read_outcar_started():
-            return
         if self.input is None:
             self.read_input_structure()
         self.read_outcar_settings()
@@ -1202,8 +1202,8 @@ class Calculation(models.Model):
             logger.info('Found no errors')
             return self
         
-        if self.label == '':
-            self.set_label(os.path.basename(self.path))
+        #if self.label == '':
+        #    self.set_label(os.path.basename(self.path))
         new_calc = self.copy()
         new_calc.set_label(self.label)
         self.set_label(self.label + '_%d' % self.attempt)
@@ -1240,7 +1240,7 @@ class Calculation(models.Model):
             elif err == 'hermitian':
                 new_calc.fix_hermitian()
             else:
-                print err
+                raise VaspError("Unknown VASP error code: %s", err)
         return new_calc
 
     def compress(self):
@@ -1251,7 +1251,7 @@ class Calculation(models.Model):
     def copy(self):
         new = Calculation()
         new.entry = self.entry
-        new.label = ''
+        new.label = self.label
         new.input = self.input
         new.output = self.output
         new.dos = self.dos
@@ -1269,13 +1269,15 @@ class Calculation(models.Model):
         os.system('cp %s/* %s 2> /dev/null' % (self.path, path))
         os.system('rm %s/* 2> /dev/null' % self.path)
         self.path = path
-        self.save()
+        if self.id:
+            Calculation.objects.filter(id=self.id).update(path=path)
 
     def backup(self):
         new_dir = '%s_' % self.attempt
         new_dir += '_'.join(self.errors)
         new_dir = new_dir.replace(' ','')
-        logger.info('backing up %s to %s' % (self.path, new_dir))
+        logger.info('backing up %s to %s' % 
+                (self.path.replace(self.entry.path+'/', ''), new_dir))
         self.move(self.path+'/'+new_dir)
 
     def clean_start(self):
@@ -1398,6 +1400,8 @@ class Calculation(models.Model):
         self.label = label
         if not self.entry is None:
             self.entry.calculations[label] = self
+        if self.id:
+            Calculation.objects.filter(id=self.id).update(label=label)
 
     def set_hubbards(self, convention='wang'):
         hubs = HUBBARDS.get(convention, {})
@@ -1436,6 +1440,11 @@ class Calculation(models.Model):
                                               us=pot_set['us'],
                                               paw=pot_set['paw'])
 
+        for e in self.elements:
+            if not e.symbol in pot_set['elements']:
+                raise VaspError('Structure contains %s, which does not have'
+                        'a potential in VASP' % e.symbol)
+
         pnames = [ pot_set['elements'][e.symbol] for e in self.elements ]
         self.potentials = list(potentials.filter(name__in=pnames))
 
@@ -1512,7 +1521,7 @@ class Calculation(models.Model):
                     new_path = '%s/%s' % (source, f)
                     self.set_chgcar(new_path)
         else:
-            logger.debug('Copying: %s to %s', source, self.path)
+            logger.debug('copying %s to %s', source, self.path)
             subprocess.check_call(['cp', source, self.path])
 
     @property
@@ -1537,6 +1546,7 @@ class Calculation(models.Model):
             formation.delta_e = self.energy_pa - e
             formation.composition = self.input.composition
             formation.entry = self.entry
+            formation.calculation = self
             self.formation = formation
             return formation
         hub_mus = chem_pots[reference]['hubbards']
@@ -1550,20 +1560,80 @@ class Calculation(models.Model):
         formation.delta_e = ( self.energy + adjust ) / self.natoms
         formation.composition = self.input.composition
         formation.entry = self.entry
+        formation.calculation = self
         self.formation = formation
         return formation
 
     @staticmethod
-    def setup(input=None, configuration='standard', path=None, entry=None,
+    def setup(structure, configuration='static', path=None, entry=None,
             hubbard='wang', potentials='vasp_rec', settings={}, 
             chgcar=None, wavecar=None,
             **kwargs):
-        
+        """
+        Method for creating a new VASP calculation.
+
+        Arguments:
+            structure: :mod:`~qmpy.Structure` instance, or string indicating an
+            input structure file.
+
+        Keyword Arguments:
+            configuration: 
+                String indicating the type of calculation to
+                perform. Options can be found with qmpy.VASP_SETTINGS.keys().
+                Create your own configuration options by adding a new file to
+                configuration/vasp_settings/inputs/ using the files already in
+                that directory as a guide. Default="static"
+
+            settings:
+                Dictionary of VASP settings to be applied to the calculation.
+                Is applied after the settings which are provided by the
+                `configuration` choice. 
+
+            path: 
+                Location at which to perform the calculation. If the
+                calculation takes repeated iterations to finish successfully,
+                all steps will be nested in the `path` directory.
+
+            entry:
+                If the full qmpy data structure is being used, you can specify
+                an entry to associate with the calculation.
+
+            hubbard:
+                String indicating the hubbard correctionconvention. Options 
+                found with qmpy.HUBBARDS.keys(), and can be added to or
+                altered by editing configuration/vasp_settings/hubbards.yml.
+                Default="wang".
+
+            potentials:
+                String indicating the vasp potentials to use. Options can be 
+                found with qmpy.POTENTIALS.keys(), and can be added to or
+                altered by editing configuration/vasp_settings/potentials/yml.
+                Default="vasp_rec".
+
+            chgcar/wavecar:
+                Calculation, or path, indicating where to obtain an initial
+                CHGCAR/WAVECAR file for the calculation.
+        """
+
+        if isinstance(structure, basestring):
+            structure = os.path.abspath(structure)
+            if path is None:
+                path = os.path.dirname(structure)
+            structure = io.read(structure, **kwargs)
+
+        # Where to do the calculation
         if path is None:
-            path = os.path.abspath(entry.path)
+            if entry is None:
+                path = os.path.abspath('.')
+            else:
+                if entry.path is None:
+                    path = os.path.abspath('.')
+                else:
+                    path = os.path.abspath(entry.path)
         else:
             path = os.path.abspath(path)
 
+        # Has the specified calculation already been created?
         if Calculation.objects.filter(path=path).exists():
             calc = Calculation.objects.get(path=path)
         else:
@@ -1576,13 +1646,13 @@ class Calculation(models.Model):
                 calc.set_chgcar(chgcar)
             if wavecar:
                 calc.set_wavecar(wavecar)
-            calc.input = input
+            calc.input = structure
             calc.kwargs = kwargs
-            if entry:
-                calc.entry = entry
+            calc.entry = entry
 
+        # What settings to use?
         if configuration not in VASP_SETTINGS:
-            raise NotImplementedError('That configuration does not exist!')
+            raise ValueError('That configuration does not exist!')
 
         calc.input.make_primitive()
 
@@ -1606,7 +1676,7 @@ class Calculation(models.Model):
                 'lscalu': False,
                 'lplane': True})
 
-        #= Error handling stuff =#
+        # Has the calculation been run?
         try:
             calc.get_outcar()
         except VaspError:
@@ -1617,20 +1687,22 @@ class Calculation(models.Model):
                 calc.write()
             return calc
 
+        # Read all outputs
         calc.read_outcar()
         calc.read_doscar()
         calc.read_stdout()
 
+        # Did the calculation finish without errors?
         if calc.converged:
             return calc
         elif not calc.errors:
             calc.write()
             return calc
-        
+
+        # Could the errors be fixed?
         fixed_calc = calc.address_errors()
         if fixed_calc.errors:
-            # unable to fix errors
-            raise VaspError("Unable to fix errors: %s", fixed_calc.errors)
+            raise VaspError('Unable to fix errors: %s' % fixed_calc.errors)
 
         fixed_calc.set_magmoms(calc.settings.get('magnetism', 'ferro'))
         fixed_calc.clear_results()
