@@ -55,10 +55,13 @@ class DOS(models.Model):
 
     @classmethod
     def read(cls, doscar='DOSCAR', efermi=0.0):
-        dos = DOS(file=os.path.abspath(doscar))
-        dos._efermi = 0.0
-        dos.read_doscar(dos.file)
-        dos.efermi = efermi
+        try:
+            dos = DOS(file=os.path.abspath(doscar))
+            dos._efermi = 0.0
+            dos.read_doscar(dos.file)
+            dos.efermi = efermi
+        except ValueError:
+            raise VaspError('Could not parse DOSCAR')
         return dos
 
     @property
