@@ -94,12 +94,18 @@ class Entry(models.Model):
                 self.reference.save()
                 self.reference = self.reference
         super(Entry, self).save(*args, **kwargs)
-        for k, v in self.structures.items():
-            v.label = k
-        self.structure_set = self.structures.values()
-        for k, v in self.calculations.items():
-            v.label = k
-        self.calculation_set = self.calculations.values()
+        if self._structures:
+            for k, v in self.structures.items():
+                v.label = k
+                v.entry = self
+                v.save()
+            #self.structure_set = self.structures.values()
+        if self._calculations:
+            for k, v in self.calculations.items():
+                v.label = k
+                v.entry = self
+                v.save()
+            #self.calculation_set = self.calculations.values()
         if self._elements:
             self.element_set = self.elements
         if self._species:
@@ -156,8 +162,8 @@ class Entry(models.Model):
 
         # Step 3
         c1 = structure.composition
-        c2 = structure.reported_composition
         if 'cif' in source_file:
+            c2 = structure.reported_composition
             if not c1.compare(c2, 5e-2):
                 entry.add_hold("composition mismatch in cif")
                 entry.composition = c2
@@ -181,15 +187,15 @@ class Entry(models.Model):
         return entry
 
     @staticmethod
-    def get(structure):
+    def get(structure, tol=1e-1):
         if isinstance(structure, Structure):
-            return Entry.search_by_structure(structure)
+            return Entry.search_by_structure(structure, tol=tol)
 
     @staticmethod
-    def search_by_structure(structure):
+    def search_by_structure(structure, tol=1e-1):
         c = Composition.get(structure.comp)
         for e in c.entries:
-            if e.structure == structure:
+            if e.structure.compare(structure, tol=tol):
                 return e
         return None
 
@@ -341,7 +347,7 @@ class Entry(models.Model):
     @property
     def unit_comp(self):
         """Composition dictionary, normalized to 1 atom."""
-        return normalize_comp(self.comp)
+        return unit_comp(self.comp)
 
     @property
     def red_comp(self):
@@ -418,7 +424,6 @@ class Entry(models.Model):
         """
         If the structure has been relaxed, returns the formation energy of the
         final relaxed structure. Otherwise, returns None.
-
         """
         if self._energy is None:
             if 'static' in self.calculations:
@@ -431,6 +436,14 @@ class Entry(models.Model):
                     self._energy = de.delta_e
         return self._energy
 
+    @property
+    def stable(self):
+        forms = self.formationenergy_set.filter(fit='standard')
+        if not forms.exists():
+            return None
+        return any([ f.stability <= 0 for f in forms ])
+        
+
     _history = None
     @property
     def history_graph(self):
@@ -548,6 +561,7 @@ class Entry(models.Model):
     def wipe(self):
         self.structure_set.exclude(label='input').delete()
         self.calculation_set.all().delete()
+        self.task_set.update(state=0)
 
     def reset(self):
         """
