@@ -148,6 +148,7 @@ class Structure(models.Model, object):
     class Meta:
         app_label = 'qmpy'
         db_table = 'structures'
+        unique_together = ('entry', 'label')
 
     def __eq__(self, other):
         return self.compare(other)
@@ -324,12 +325,15 @@ class Structure(models.Model, object):
         self.label = label
         if not self.entry is None:
             self.entry.structures[label] = self
+        if self.id:
+            Structure.objects.filter(id=self.id).update(label=label)
 
     def set_volume(self, value):
         """
         Rescales the unit cell to the specified volume, keeping the direction
         and relative magnitudes of all lattice vectors the same.
         """
+        self.get_volume()
         scale = value/self.volume
         self.cell = self.cell * scale**(1/3.)
         self.volume_pa = value/self.natoms
@@ -552,9 +556,12 @@ class Structure(models.Model, object):
                     return False
         return True
 
-    def compare(self, other, tol=0.1, volume=False, 
-                                       allow_distortions=False, 
-                                       wildcard=None):
+    def compare(self, other, tol=0.01,
+                             atom_tol=10,
+                             volume=False, 
+                             allow_distortions=False, 
+                             check_spacegroup=False,
+                             wildcard=None):
         """
         Credit to K. Michel for the algorithm.
 
@@ -567,6 +574,8 @@ class Structure(models.Model, object):
         4. Check that the number of atoms of each element are the same in
         primitive cells
 
+        4b. Check that the spacegroup is the same.
+
         5. If needed check that the primitive cell volumes are the same
 
         6. Convert both primitive cells to reduced form There is one issue here - 
@@ -606,8 +615,8 @@ class Structure(models.Model, object):
         """
 
         # 1
-        if len(self) > 80 or len(other) > 80:
-            return False
+        #if len(self) > 80 or len(other) > 80:
+        #    return False
         me = self.copy()
         you = other.copy()
 
@@ -640,6 +649,13 @@ class Structure(models.Model, object):
         me.reduce()
         you.reduce()
 
+        #6b
+        if check_spacegroup:
+            me.symmetrize()
+            you.symmetrize()
+            if me.spacegroup != you.spacegroup:
+                return False
+
         # 7
         try_again = False
         for a, b in zip(me.lat_params[3:], you.lat_params[3:]):
@@ -685,6 +701,9 @@ class Structure(models.Model, object):
 
         test_struct = you.copy()
 
+        eps = 2*tol*atom_tol#*me.volume**(1./3)
+        eps2 = eps**2
+
         for rot in rotations:
             # loop over all possible re-orientations of the cell
             inv = la.inv(rot)
@@ -696,15 +715,47 @@ class Structure(models.Model, object):
                     continue
 
                 test_struct.coords -= test_struct[i].coord
-                # check 
+                # check if all sites have a match
                 match = True
+                matches = []
+                vecs = []
                 for atom2 in test_struct:
-                    if not me.contains(atom2, tol=tol*10):
+                    best = 1000
+                    id = None
+                    vec = None
+                    for j, atom3 in enumerate(me):
+                        if j in matches:
+                            continue
+                        if atom2.element_id != atom3.element_id:
+                            continue
+                        d = me._get_vector(atom2, atom3)
+                        if any([ abs(dd) > eps for dd in d ]):
+                            continue
+                        d2 = d.dot(d)
+                        if d2 > eps2:
+                            continue
+
+                        # matching case
+                        if d2 < best:
+                            best = d2
+                            id = j
+                            vec = d
+
+                    if id is None:
                         match = False
                         break
+                    matches.append(id)
+                    vecs.append(vec)
 
-                if match:
+                if match == False:
+                    continue
+                vecs = np.array(vecs)
+                err = np.average(vecs, 0)
+                vecs -= err
+                if all([ d.dot(d)**0.5 < tol*atom_tol for d in vecs ]):
                     return True
+                #else:
+                #    print vecs
 
         logger.debug("Atoms don't match.")
         return False
@@ -786,6 +837,22 @@ class Structure(models.Model, object):
 
         return dist
 
+    def _get_vector(self, atom1, atom2):
+
+        x, y, z = self.cell
+        xx = self.metrical_matrix[0,0]
+        yy = self.metrical_matrix[1,1]
+        zz = self.metrical_matrix[2,2]
+
+        vec = atom2.coord - atom1.coord
+        vec -= np.round(vec)
+        dist = np.dot(vec, self.cell)
+
+        dist -= round(dist.dot(x)/xx)*x
+        dist -= round(dist.dot(y)/yy)*y
+        dist -= round(dist.dot(z)/zz)*z
+        return dist
+
     def add_atom(self, atom, tol=0.01):
         """
         Adds `atom` to the structure if it isn't already contained.
@@ -930,6 +997,11 @@ class Structure(models.Model, object):
         """Return atomic positions in cartesian coordinates."""
         return np.array([ atom.cart_coord for atom in self.atoms ]) 
 
+    @cartesian_coords.setter
+    def cartesian_coords(self, cc):
+        for atom, coord in zip(self.atoms, cc):
+            atom.cart_coord = coord
+
     @property
     def forces(self):
         """numpy.ndarray of forces on atoms."""
@@ -980,7 +1052,7 @@ class Structure(models.Model, object):
         while self.natoms*np.product(kpts) < kppra:
             prev_kpts = kpts.copy()
             refk = np.array(np.ones(3)*refr)*scale
-            kpts = np.array(map(np.ceil, refk))
+            kpts = np.array(map(np.round, refk))
             scale += 1
 
         upper = kppra - np.product(prev_kpts)*self.natoms
@@ -1152,7 +1224,7 @@ class Structure(models.Model, object):
                 return False
         return True
 
-    def find_nearest_neighbors(self, method='closest', tol=0.1, limit=5.0):
+    def find_nearest_neighbors(self, method='closest', tol=0.05, limit=5.0):
         """
         Calls :func:`~qmpy.get_nearest_neighbors()`  and assigns the nearest
         neighbor dictionary to `Structure._nearest_neighbors.
@@ -1203,16 +1275,11 @@ class Structure(models.Model, object):
         Examples::
             
             >>> s = io.read(INSTALL_PATH+'/io/files/fe3o4.cif')
-            >>> spin_lattice = s.get_spin_lattice(elements=['Fe'])
+            >>> sl = s.get_spin_lattice(elements=['Fe'])
             >>> sl.set_fraction(0.33333)
             >>> sl.fraction
             0.3333333333333333
             >>> sl.run_MC()
-            >>> sl.up_spins
-            [<Atom: Fe @ 0.5 0.0 0.0>, 
-             <Atom: Fe @ 0.0 0.5 0.0>, 
-             <Atom: Fe @ 0.0 0.0 0.5>, 
-             <Atom: Fe @ 0.5 0.5 0.5>]
 
         """
 
@@ -1224,10 +1291,14 @@ class Structure(models.Model, object):
             return struct.get_spin_lattice()
         self.find_nearest_neighbors()
         for a1 in self.atoms:
+            n0 = len(pairs)
             for a2 in a1.neighbors:
-                pairs.append([a1, a2])
+                #if a1 < a2:
+                #    continue
+                pairs.append((a1, a2))
 
         lattice = SpinLattice(pairs)
+        lattice.structure = self
         return lattice
 
     def displace_atom(structure, index, vector):
@@ -1265,7 +1336,7 @@ class Structure(models.Model, object):
             return new
 
         def disp():
-            dists = np.array(self.lp[:3])*distance
+            dists = np.array([ distance/i for i in self.lp[:3]])
             rands = [ random.random() for i in range(3) ]
             return dists*rands
 
@@ -1363,10 +1434,12 @@ class Structure(models.Model, object):
 
         """
         limits = [ int(np.ceil(distance/self.lp[i])) for i in range(3) ]
-        ranges = [ range(-l, l+1) for l in limits ]
+        ranges = [ range(0, l+1) for l in limits ]
         d2 = distance**2
         lattice_points = []
         for i, j, k in itertools.product(*ranges):
+            #if (i,j,k) == (0,0,0):
+            #    continue
             point = np.dot([i,j,k], self.cell)
             if any([ abs(p) > distance for p in point ]):
                 continue
@@ -1519,14 +1592,15 @@ class Structure(models.Model, object):
 
         if not in_place:
             new = self.copy()
-            new.substitute(replace, rescale=rescale, in_place=False)
+            new.substitute(replace, rescale=rescale, in_place=True)
             return new
 
-        volume = 0.0
+        volume = self.get_volume()
         for atom in self:
             if atom.element_id in replace:
-                atom.element_id = replace[atom.element_id]
-            volume += atom.element.volume
+                volume -= atom.element.volume
+                atom.element = Element.get(replace[atom.element_id])
+                volume += atom.element.volume
         if rescale:
             self.set_volume(volume)
         self.set_composition()
@@ -1754,6 +1828,38 @@ class Structure(models.Model, object):
             return False
         return True
 
+    def create_slab(self, indices, vacuum=10.0, surface=None, in_place=True):
+        if not in_place:
+            new = self.copy()
+            return new.create_slab(indices, vacuum=vacuum, surface=surface)
+
+    def create_vacuum(self, direction, amount, in_place=True):
+        """
+        Add vacuum along a lattice direction.
+
+        Arguments:
+            direction: direction to add the vacuum along. (0=x, 1=y, 2=z)
+            amount: amount of vacuum in Angstroms.
+
+        Keyword Arguments:
+            in_place: apply change to current structure, or return a new one.
+
+        Examples::
+            
+            >>> s = io.read(INSTALL_PATH+'/io/files/POSCAR_FCC')
+            >>> s.create_vacuum(2, 5)
+        """
+        if not in_place:
+            new = self.copy()
+            return new.create_vacuum(direction, amount)
+
+        cart_coords = self.cartesian_coords
+        new_cell = self.lat_params
+        new_cell[direction] += float(amount)
+        self.lat_params = new_cell
+        self.cartesian_coords = cart_coords
+        return self
+
     def make_perfect(self, in_place=True, tol=1e-1):
         """
         Constructs options for a 'perfect' lattice from the structure.
@@ -1787,7 +1893,7 @@ class Structure(models.Model, object):
         """
         if not in_place:
             new = self.copy()
-            return new.make_perfect(True)
+            return new.make_perfect(True, tol=tol)
 
         init_atoms = [ a.copy() for a in self.atoms ]
         init_comp = dict(self.comp)
@@ -1810,13 +1916,11 @@ class Structure(models.Model, object):
                 new.occupancy = 1.0
                 new.site = None
                 atoms.append(new)
-            elif abs(atom.occupancy) < 0.5:
+            elif abs(atom.occupancy) <= 0.5:
                 # then, empty any nearly empty sites
                 continue
             elif atom.occupancy >= 2:
                 raise StructureError('Site occupied by a molecule')
-            else:
-                continue
 
         self._sites = []
         self.atoms = atoms
@@ -1824,12 +1928,12 @@ class Structure(models.Model, object):
 
         if self.is_perfect:
             # total
-            if abs(1 - sum(self.comp.values())/n) > tol:
+            if abs(sum(self.comp.values()) - n) > tol:
                 hopeless = True
 
             for k in init_comp.keys():
-                d = init_comp[k] - self.comp.get(k,0.0)
-                if abs(d) > n*tol:
+                d = init_comp[k] - self.comp.get(k, 0.0)
+                if abs(d) > tol:
                     hopeless = True
                     break
             self.composition = Composition.get(self.comp)
